/*
  UsingMutexLocks_Ethernet

  The sketch is almost identical to BrowseRoot_WiFi/_Ethernet. It prints 
  the root content of a media server.
	
  The difference is that we create more than one thread and use a project
  wide mutex lock to avoid problems as explained in Readme.md. 
  This example gives you an idea how to do it.
	
  We use a Wiznet W5x00 Ethernet module/shield instead of builtin WiFi.
  It's connected to ESP32 GPIO 18, 19, 23 and GPIO 25 (Chip Select).

  Last updated 2021-02-02, ThJ <yellobyte@bluewin.ch>
*/

#include <Arduino.h>
#include <Ethernet.h>
#include "SoapESP32.h"

// == IMPORTANT ==
// We use Ethernet module/shield instead of WiFi, so you must do one of the following:
// 1) uncomment line "//#define USE_ETHERNET" in SoapESP32.h OR
// 2) add -DUSE_ETHERNET to compiler.cpreprocessor.flags in platform.txt (ArduinoIDE) OR
// 3) add -DUSE_ETHERNET to your build_flags in platformio.ini (VSCode/PlatformIO)

// uncomment in case you want to know
//#define SHOW_ESP32_MEMORY_STATISTICS

// MAC address for your Ethernet module/shield
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };

EthernetClient client;
EthernetUDP    udp;

const int led = 2;                                		// LED pin on ESP32 board
int8_t ledStat = 0;                                 	// counter needed to let LED blink
TaskHandle_t      xAnotherTask;                     	// task handle for an additional task
SemaphoreHandle_t SPIsem = NULL;                   	// smeaphore used as global mutex
SoapESP32         soap(&client, &udp, &SPIsem);

// example for another task using the SPI bus
void anotherTask (void *parameter)
{
  while (true) {
    while (xSemaphoreTake(SPIsem, 10) != pdTRUE) {};    // claim SPI bus
    // do something, e.g. SD writes/reads via SPI
    delayMicroseconds(200);                             // make the other tasks wait
    xSemaphoreGive(SPIsem);                             // release SPI bus

    // just needed for this example: yields processor and resets task switcher watchdog 
    delay(1);

    // toggle LED to show this task is still alive
    digitalWrite(led, (++ledStat > 0) ? 1 : 0);

    while (xSemaphoreTake(SPIsem, 10) != pdTRUE) {};    // claim SPI bus
    // do something, e.g. write audio data to codec via SPI
    delayMicroseconds(200);                             // make the other tasks wait
    xSemaphoreGive(SPIsem);                             // release SPI bus
    //
    // etc...
    //
  }
}

void setup() {
  Serial.begin(115200);

  Ethernet.init(25);    // CS to ESP32 GPIO 25
  Serial.print("\nInitializing Ethernet...");

  if (Ethernet.begin(mac))
  {
    Serial.println("DHCP ok.");
  }
  else
  {
    Serial.println("DHCP error !");
    while (true) {
      // no point to continue
    }
  }

  Serial.print("Local IP: ");
  Serial.println(Ethernet.localIP());
	
  // initial stuff
  SPIsem = xSemaphoreCreateMutex();
  pinMode(led, OUTPUT);

  // start another task that will use SPI as well
  xTaskCreate(
    anotherTask,                       // pointer to task defined above
    "anotherTask",                     // name of task.
    1024,                              // Stack size of task
    NULL,                              // parameter for the task
    1,                                 // priority of the task
    &xAnotherTask);                    // task handle to keep track of it
		
  Serial.println("Additional task started. LED should blink.");

  // scan local network for DLNA media servers
  Serial.println();
  Serial.println("Scanning local network for DLNA media servers...");
  soap.seekServer();
  Serial.print("Number of discovered servers that deliver content: ");
  Serial.println(soap.getServerCount());
  Serial.println();

  // Show root content of all discovered, usable media servers
  soapObjectVect_t browseResult;       // browse results get stored here
  soapServer_t serv;                   // single server info gets stored here
  int i = 0;                           // start with first entry in server list

  while (soap.getServerInfo(i, &serv)) {
    // Print some server details
    Serial.print("Server[");
    Serial.print(i);
    Serial.print("]: IP address: ");
    Serial.print(serv.ip);
    Serial.print(", port: ");
    Serial.print(serv.port);
    Serial.print(", name: ");
    Serial.println(serv.friendlyName);

    // browse root (always represented by "0" according to spec)
    if (!soap.browseServer(i, "0", &browseResult)) {
      Serial.println("error browsing server.");
    }
    else {
      Serial.print("Browsing root directory. Number of sub-directories: ");
      Serial.println(browseResult.size());

      // show each directory in root
      for (int j = 0; j < browseResult.size(); j++) {
        Serial.print(" ");
        Serial.print(browseResult[j].name);
        if (browseResult[j].isDirectory) {
          Serial.print(" (child count: ");
        }
        else {
          // root shouldn't host files so it's unlikely we get here
          Serial.print(" (File, Size: ");
        }
        Serial.print(browseResult[j].size);
        Serial.println(")");
      }
    }
    Serial.println("");
    i++;
  }

#ifdef SHOW_ESP32_MEMORY_STATISTICS
  Serial.println("Some ESP32 memory stats after running this sketch:");
  Serial.print(" 1) minimum ever free memory of all regions [in bytes]: ");
  Serial.println(ESP.getMinFreeHeap());
  Serial.print(" 2) minimum ever free heap size [in bytes]:             ");
  Serial.println(xPortGetMinimumEverFreeHeapSize());
  Serial.print(" 3) minimum ever stack size of this task [in bytes]:    ");
  Serial.println(uxTaskGetStackHighWaterMark(NULL)); 
#endif

  Serial.println();
  Serial.println("Sketch finished.");
}

void loop() {
  // 
}
